#programme qui détecte les keypoints d'une image avec orb, les affiches sur l'image ainsi que leurs coordonnées
#et stocke ces coordonnées dans un fichier nommé cd.txt
#Il procéde ensuite à effectuer la triangulation de Delaunay et a afficher le diagramme de voroinoi correspondant
import numpy as np
import cv2
from matplotlib import pyplot as plt
import random

# Check if a point is inside a rectangle
def rect_contains(rect, point) :
    if point[0] < rect[0] :
        return False
    elif point[1] < rect[1] :
        return False
    elif point[0] > rect[2] :
        return False
    elif point[1] > rect[3] :
        return False
    return True

# Draw a point
def draw_point(img, p, color ) :
    cv2.circle( img, p, 2, color, cv2.FILLED, cv2.LINE_AA, 0 )


# Draw delaunay triangles
def draw_delaunay(img, subdiv, delaunay_color ) :

    triangleList = subdiv.getTriangleList();
    size = img.shape
    r = (0, 0, size[1], size[0])
    for t in triangleList :
        
        pt1 = (t[0], t[1])
        pt2 = (t[2], t[3])
        pt3 = (t[4], t[5])

        
        if rect_contains(r, pt1) and rect_contains(r, pt2) and rect_contains(r, pt3) :
            tri=[]
            cv2.line(img, pt1, pt2, delaunay_color, 1, cv2.LINE_AA, 0)
            cv2.line(img, pt2, pt3, delaunay_color, 1, cv2.LINE_AA, 0)
            cv2.line(img, pt3, pt1, delaunay_color, 1, cv2.LINE_AA, 0)
            tri.append((pt1,pt2,pt3))
            print(tri)

# Draw voronoi diagram
def draw_voronoi(img, subdiv) :

    ( facets, centers) = subdiv.getVoronoiFacetList([])

    for i in range(0,len(facets)) :
        ifacet_arr = []
        for f in facets[i] :
            ifacet_arr.append(f)
        
        ifacet = np.array(ifacet_arr, np.int)
        color = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))

        cv2.fillConvexPoly(img, ifacet, color, cv2.LINE_AA, 0);
        ifacets = np.array([ifacet])
        cv2.polylines(img, ifacets, True, (0, 0, 0), 1, cv2.LINE_AA, 0)
        cv2.circle(img, (centers[i][0], centers[i][1]), 3, (0, 0, 0), cv2.FILLED, cv2.LINE_AA, 0)
img = cv2.imread('/home/pi/stage_matthieu_alban/rotacon60pr100.bmp',0)#chemin de l'image a analysé

# Initiate STAR detector
orb = cv2.ORB_create()

# find the keypoints with ORB
kp = orb.detect(img,None)

# compute the descriptors with ORB
kp, des = orb.compute(img, kp)

# draw only keypoints location,not size and orientation
img2 = cv2.drawKeypoints(img,kp,None,color=(0,255,0), flags=0)

n= len(kp)
cd_x = np.zeros(n, dtype=np.int)
cd_y = np.zeros(n, dtype=np.int)
for i in range (0, n):
	cd_x[i] = kp[i].pt[0]
	cd_y[i] = kp[i].pt[1]

print(cd_x)
print("\n\n")
print(cd_y)
print("\n\n")

cd = []
cd = np.column_stack((cd_x,cd_y))
print(cd)
np.savetxt("/home/pi/stage_matthieu_alban/cd.txt", cd, fmt='%i')

plt.imshow(img2),plt.show(),plt.imsave("rotacon80pr100_KP.jpg", img2)#sauvegarde l'image avec les KP dessinés dessus

# Define window names
win_delaunay = "Delaunay Triangulation"
win_voronoi = "Voronoi Diagram"

# Turn on animation while drawing triangles
animate = True
    
# Define colors for drawing.
delaunay_color = (255,255,255)
points_color = (0, 0, 255)
    
# Keep a copy around
img_orig = img.copy();
    
# Rectangle to be used with Subdiv2D
size = img.shape
rect = (0, 0, size[1], size[0])
    
# Create an instance of Subdiv2D
subdiv = cv2.Subdiv2D(rect);

# Create an array of points.
points = [];
    
# Read in the points from a text file
with open("/home/pi/stage_matthieu_alban/cd.txt") as file :
    for line in file :
        x, y = line.split()
        points.append((int(x), int(y)))
    
# Insert points into subdiv
for p in points :
    subdiv.insert(p)
    # Show animation
    if animate :
        img_copy = img_orig.copy()
        # Draw delaunay triangles
        draw_delaunay( img_copy, subdiv, (255, 255, 255) );
        cv2.imshow(win_delaunay, img_copy)
        cv2.waitKey(100)

# Draw delaunay triangles
draw_delaunay( img, subdiv, (255, 255, 255) );

# Draw points
for p in points :
    draw_point(img, p, (0,0,255))
# Allocate space for voronoi Diagram
img_voronoi = np.zeros(img.shape, dtype = img.dtype)

# Draw voronoi diagram
draw_voronoi(img_voronoi,subdiv)
# Show results
cv2.imshow(win_delaunay,img)
cv2.imshow(win_voronoi,img_voronoi)
cv2.waitKey(0)






